package me.unurled.sacredrealms.sr.commands.admin;

import static me.unurled.sacredrealms.sr.utils.Component.NOT_PLAYER;
import static me.unurled.sacredrealms.sr.utils.Component.textComp;

import java.util.List;
import java.util.Set;
import me.unurled.sacredrealms.sr.components.treasure.Treasure;
import me.unurled.sacredrealms.sr.components.treasure.TreasureManager;
import me.unurled.sacredrealms.sr.gui.treasure.TreasureGUI;
import me.unurled.sacredrealms.sr.managers.Manager;
import net.kyori.adventure.text.Component;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabExecutor;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import xyz.xenondevs.invui.window.Window;

public class TreasureCommand implements TabExecutor {

  public static final Set<Material> TRANSPARENT =
      Set.of(Material.AIR, Material.CAVE_AIR, Material.VOID_AIR, Material.WATER, Material.LAVA);
  public static final String SR_TREASURE_PERMISSION = "sr.treasure.";
  public static final String CREATE = "create";
  public static final String MODIFY = "modify";
  public static final String LIST = "list";
  public static final String DELETE = "delete";

  private static void delete(@NotNull CommandSender sender, TreasureManager instance) {
    Block block;
    if (!(sender instanceof Player p)) {
      sender.sendMessage(NOT_PLAYER);
      return;
    }
    block = p.getTargetBlock(TRANSPARENT, 3);
    if (block.getBlockData().getMaterial().equals(Material.CHEST)) {
      // find the treasure chest
      Treasure treasure = instance.findByLocation(block.getLocation());
      if (treasure == null) {
        sender.sendMessage("No treasure chest found at this location.");
        return;
      }
      // remove the treasure chest
      instance.removeTreasure(treasure);
    }
  }

  private static void list(@NotNull CommandSender sender, @NotNull TreasureManager instance) {
    sender.sendMessage("Treasures:");
    for (Treasure treasure : instance.getTreasures()) {
      String s =
          String.format(
              "<click:run_command:tp %s %s %s>%s</click>x: %s, y: %s, z: %s",
              treasure.getLocation().getBlockX(),
              treasure.getLocation().getBlockY(),
              treasure.getLocation().getBlockZ(),
              treasure.getLocation().toString(),
              treasure.getLocation().getBlockX(),
              treasure.getLocation().getBlockY(),
              treasure.getLocation().getBlockZ());
      Component text = textComp(s);
      sender.sendMessage(text);
    }
  }

  private static void modify(@NotNull CommandSender sender, TreasureManager instance) {
    Block block;
    if (!(sender instanceof Player p)) {
      sender.sendMessage(NOT_PLAYER);
      return;
    }
    block = p.getTargetBlock(TRANSPARENT, 3);
    if (block.getBlockData().getMaterial().equals(Material.CHEST)) {
      // find the treasure chest
      Treasure treasure = instance.findByLocation(block.getLocation());
      if (treasure == null) {
        sender.sendMessage("No treasure chest found at this location.");
        return;
      }
      // open the GUI
      Window window =
          Window.single()
              .setViewer(p)
              .setTitle("Attributes")
              .setGui(TreasureGUI.createGui(treasure))
              .build();
      window.open();
    }
  }

  private static void create(
      @NotNull CommandSender sender, @NotNull String @NotNull [] args, TreasureManager instance) {
    Block block;
    if (!(sender instanceof Player p)) {
      sender.sendMessage(NOT_PLAYER);
      return;
    }
    block = p.getTargetBlock(TRANSPARENT, 3);
    if (block.getBlockData().getMaterial().equals(Material.CHEST)) {
      // create a new treasure chest
      Treasure treasure;
      if (args.length == 2) {
        if (!args[1].startsWith(SR_TREASURE_PERMISSION)) {
          p.sendMessage(textComp("<red>Invalid permission node. It must start with sr.treasure."));
          return;
        }
        treasure = new Treasure(instance.getNextId(), block.getLocation(), args[1]);
      } else {
        treasure = new Treasure(instance.getNextId(), block.getLocation());
      }
      instance.addTreasure(treasure);
    }
  }

  /**
   * Executes the given command, returning its success. <br>
   * If false is returned, then the "usage" plugin.yml entry for this command (if defined) will be
   * sent to the player.
   *
   * @param sender Source of the command
   * @param command Command which was executed
   * @param label Alias of the command which was used
   * @param args Passed command arguments
   * @return true if a valid command, otherwise false
   */
  @Override
  public boolean onCommand(
      @NotNull CommandSender sender,
      @NotNull Command command,
      @NotNull String label,
      @NotNull String @NotNull [] args) {
    // create, modify, list, get, delete
    if (args.length == 0) {
      sender.sendMessage("Usage: /treasure <create|modify|list|delete>");
      return true;
    }
    final TreasureManager instance = Manager.getInstance(TreasureManager.class);
    switch (args[0]) {
      case CREATE -> create(sender, args, instance);
      case MODIFY -> modify(sender, instance);
      case LIST -> list(sender, instance);
      case DELETE -> delete(sender, instance);
      default -> sender.sendMessage("Usage: /treasure <create|modify|list|delete>");
    }
    return true;
  }

  /**
   * Requests a list of possible completions for a command argument.
   *
   * @param sender Source of the command. For players tab-completing a command inside of a command
   *     block, this will be the player, not the command block.
   * @param command Command which was executed
   * @param label Alias of the command which was used
   * @param args The arguments passed to the command, including final partial argument to be
   *     completed
   * @return A List of possible completions for the final argument, or null to default to the
   *     command executor
   */
  @Override
  public @Nullable List<String> onTabComplete(
      @NotNull CommandSender sender,
      @NotNull Command command,
      @NotNull String label,
      @NotNull String @NotNull [] args) {
    if (args.length == 0) {
      return List.of(CREATE, MODIFY, LIST, DELETE);
    }
    if (args.length == 2 && (args[0].equals(CREATE))) {
      if (args[1].startsWith(SR_TREASURE_PERMISSION)) {
        return List.of();
      }
      return List.of(SR_TREASURE_PERMISSION);
    }

    return List.of();
  }
}
