package me.unurled.sacredrealms.sr.components.combat;

import static me.unurled.sacredrealms.sr.utils.Component.comp;
import static me.unurled.sacredrealms.sr.utils.SRPlayerUtils.spawnIndicator;
import static me.unurled.sacredrealms.sr.utils.SRPlayerUtils.updateActionBar;

import java.util.HashMap;
import java.util.Map.Entry;
import java.util.Random;
import java.util.UUID;
import me.unurled.sacredrealms.sr.components.attributes.Attribute;
import me.unurled.sacredrealms.sr.components.entity.EntityManager;
import me.unurled.sacredrealms.sr.components.entity.SREntity;
import me.unurled.sacredrealms.sr.components.player.PlayerManager;
import me.unurled.sacredrealms.sr.components.player.SRPlayer;
import me.unurled.sacredrealms.sr.managers.Manager;
import me.unurled.sacredrealms.sr.utils.Items;
import org.bukkit.Sound;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Mob;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.entity.EntityDamageByBlockEvent;
import org.bukkit.event.entity.EntityDamageByEntityEvent;
import org.bukkit.event.entity.EntityDamageEvent.DamageCause;
import org.bukkit.inventory.EntityEquipment;
import org.jetbrains.annotations.NotNull;

public class CombatManager extends Manager {

  public static final Random RANDOM = new Random();
  private final HashMap<UUID, HashMap<Player, Double>> historyMobDamage = new HashMap<>();

  @EventHandler
  public void onDamage(@NotNull EntityDamageByBlockEvent e) {
    // TODO: Implement
    // blast enchant :shrug:
    // fall damage, fire damage, etc
    e.setDamage(0.0);
    e.setCancelled(true);
  }

  private static void playerVictim(
      @NotNull EntityDamageByEntityEvent e, Player player, Mob mob, PlayerManager pm) {
    // get equipment of damager
    EntityEquipment equipment = mob.getEquipment();
    Double dStrength = Items.getTotalAttribute(equipment, Attribute.STRENGTH);
    Double eDefense = Items.getTotalAttribute(player, Attribute.DEFENSE);

    double damage = dStrength - eDefense;

    Double dLuck = Items.getTotalAttribute(equipment, Attribute.LUCK);
    Double eLuck = Items.getTotalAttribute(player, Attribute.LUCK);

    double luck = dLuck - eLuck;

    if (luck < -1000) {
      // 100% chance of miss
      player.sendMessage(comp("<red>You dodged!"));
      player.playSound(player, Sound.ENTITY_ARROW_SHOOT, 1.0f, 1.0f);
      e.setCancelled(true);
      return;
    } else if (luck > 1000) {
      // 100% chance of critical hit
      luck = 10000d;
      player.playSound(player, Sound.ENTITY_PLAYER_ATTACK_CRIT, 1.0f, 1.0f);
      damage = damage * 2;
    }

    if (luck < 0) {
      // chances of miss
      if (RANDOM.nextBoolean()) {
        player.playSound(player, Sound.ENTITY_ARROW_SHOOT, 1.0f, 1.0f);
        e.setCancelled(true);
        player.sendMessage(comp("<red>You dodged!"));
        return;
      }
    } else if (luck > 0 && luck < 1000 && RANDOM.nextBoolean()) {
      // chance of critical hit
      player.playSound(player, Sound.ENTITY_PLAYER_ATTACK_CRIT, 1.0f, 1.0f);
      // damage * 2
      damage = damage * 2;
    }

    // apply damage
    player.damage(damage);
    // add an

    SRPlayer player1 = pm.getPlayer(player.getUniqueId());
    if (player1 != null) {
      player1.setHealth(player.getHealth() - damage, DamageCause.ENTITY_ATTACK);
      if (player1.getHealth() <= 0) {
        player.damage(player.getHealth());
      }
      updateActionBar(player, player1);
    }

    spawnIndicator(player, false, damage);

    // TODO: check for status effects (apply status effects depending item of damager)
  }

  // TODO: Finish this
  @EventHandler
  public void onDamage(@NotNull EntityDamageByEntityEvent e) {
    e.setDamage(0.0);

    if (!(e.getDamager() instanceof LivingEntity damager)
        || !(e.getEntity() instanceof LivingEntity entity)) {
      return;
    }
    PlayerManager pm = Manager.getInstance(PlayerManager.class);
    if (pm == null) {
      return;
    }

    if (damager instanceof Player d) {
      playerDamager(e, entity, d, pm);
      return;
    }
    if (entity instanceof Player player && damager instanceof Mob mob) {
      playerVictim(e, player, mob, pm);
    }
  }

  private void playerDamager(
      @NotNull EntityDamageByEntityEvent e, LivingEntity entity, Player d, PlayerManager pm) {
    if (entity instanceof Player) {
      // no pvp
      e.setCancelled(true);
      d.sendMessage(comp("<red>You can't attack players!"));
      d.playSound(d, Sound.BLOCK_ANVIL_HIT, 1.0f, 1.0f);
      return;
    }
    if (entity instanceof Mob mob) {
      isMob(e, entity, d, pm, mob);
    }
  }

  private void isMob(
      @NotNull EntityDamageByEntityEvent e,
      LivingEntity entity,
      Player d,
      PlayerManager pm,
      Mob mob) {
    double dStrength = Items.getTotalAttribute(d, Attribute.STRENGTH);
    SRPlayer player = pm.getPlayer(d.getUniqueId());
    if (player != null) {
      dStrength += player.getAttribute(Attribute.STRENGTH);
    }
    EntityEquipment equipment = mob.getEquipment();
    double eDefense = Items.getTotalAttribute(equipment, Attribute.DEFENSE);

    double damage = dStrength - eDefense;

    Double dLuck = Items.getTotalAttribute(d, Attribute.LUCK);
    Double eLuck = Items.getTotalAttribute(equipment, Attribute.LUCK);

    double luck = dLuck - eLuck;
    if (luck < -1000) {
      // 100% chance of miss
      d.sendMessage(comp("<red>You missed!"));
      d.playSound(d, Sound.ENTITY_ARROW_SHOOT, 1.0f, 1.0f);
      e.setCancelled(true);
      return;
    } else if (luck > 1000) {
      // 100% chance of critical hit
      luck = 10000d;
      d.playSound(d, Sound.ENTITY_PLAYER_ATTACK_CRIT, 1.0f, 1.0f);
      damage = damage * 2;
    }

    if (luck < 0) {
      // chances of miss
      if (RANDOM.nextBoolean()) {
        d.playSound(d, Sound.ENTITY_ARROW_SHOOT, 1.0f, 1.0f);
        e.setCancelled(true);
        d.sendMessage(comp("<red>You missed!"));
        return;
      }
    } else if (luck > 0 && luck < 1000 && RANDOM.nextBoolean()) {
      // chance of critical hit
      d.playSound(d, Sound.ENTITY_PLAYER_ATTACK_CRIT, 1.0f, 1.0f);
      // damage * 2
      damage = damage * 2;
    }
    // apply damage

    historyHandler(entity, d, damage);
    entity.damage(damage);

    EntityManager em = Manager.getInstance(EntityManager.class);
    SREntity ent = em.getEntity(entity.getUniqueId());

    if (entity.getHealth() <= 0) {
      die(entity, d, pm, ent);
      return;
    }

    spawnIndicator(entity, false, damage);
    // TODO: check for status effects (apply status effects depending item of damager)
  }

  private void die(LivingEntity entity, Player d, PlayerManager pm, SREntity ent) {
    // entity is dead, give experience to player
    SRPlayer player1 = pm.getPlayer(d.getUniqueId());
    if (player1 != null && ent != null) {
      // calculate experience % based on damage
      HashMap<Player, Double> playerDoubleHashMap = historyMobDamage.get(entity.getUniqueId());
      if (playerDoubleHashMap == null) {
        return;
      }
      for (Entry<Player, Double> map : playerDoubleHashMap.entrySet()) {
        Double exp = ent.getExperience() * (map.getValue() / ent.getHealth());
        SRPlayer tmp = pm.getPlayer(map.getKey().getUniqueId());
        if (tmp != null) {
          tmp.setExperience((long) (tmp.getExperience() + exp));
        }
      }
    }
  }

  private void historyHandler(LivingEntity entity, Player d, Double damage) {
    if (historyMobDamage.containsKey(entity.getUniqueId())) {
      HashMap<Player, Double> history =
          historyMobDamage.getOrDefault(entity.getUniqueId(), new HashMap<>());
      if (history.containsKey(d)) {
        history.put(d, history.get(d) + damage);
      } else {
        history.put(d, damage);
      }
    } else {
      HashMap<Player, Double> history = new HashMap<>();
      history.put(d, damage);
      historyMobDamage.put(entity.getUniqueId(), history);
    }
  }
}
