package me.unurled.sacredrealms.sr.components.item.abilities;

import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import java.lang.reflect.Type;

public class AbilityDeserializer implements JsonDeserializer<Ability> {

  /**
   * Gson invokes this call-back method during deserialization when it encounters a field of the
   * specified type.
   *
   * <p>In the implementation of this call-back method, you should consider invoking {@link
   * JsonDeserializationContext#deserialize(JsonElement, Type)} method to create objects for any
   * non-trivial field of the returned object. However, you should never invoke it on the same type
   * passing {@code json} since that will cause an infinite loop (Gson will call your call-back
   * method again).
   *
   * @param json The Json data being deserialized
   * @param typeOfT The type of the Object to deserialize to
   * @param context
   * @return a deserialized object of the specified type typeOfT which is a subclass of {@code T}
   * @throws JsonParseException if json is not in the expected format of {@code typeofT}
   */
  @Override
  public Ability deserialize(JsonElement json, Type typeOfT, JsonDeserializationContext context)
      throws JsonParseException {
    try {
      JsonObject jsonObject = json.getAsJsonObject();

      String name = jsonObject.get("name").getAsString();
      String description = jsonObject.get("description").getAsString();
      Integer cooldown = jsonObject.get("cooldown").getAsInt();
      Integer manaCost = jsonObject.get("manaCost").getAsInt();
      Integer damage = jsonObject.get("damage").getAsInt();

      return new Ability(name, description, cooldown, manaCost, damage);
    } catch (Exception e) {
      throw new JsonParseException("Error deserializing Ability: " + e.getMessage());
    }
  }
}
