package me.unurled.sacredrealms.sr.components.treasure;

import static me.unurled.sacredrealms.sr.utils.Logger.error;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonParseException;
import io.leangen.geantyref.TypeToken;
import java.lang.reflect.Type;
import java.util.Map;
import me.unurled.sacredrealms.sr.components.item.ItemStackDeserializer;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.inventory.ItemStack;

public class TreasureDeserializer implements JsonDeserializer<Treasure> {

  /**
   * Gson invokes this call-back method during deserialization when it encounters a field of the
   * specified type.
   *
   * <p>In the implementation of this call-back method, you should consider invoking {@link
   * JsonDeserializationContext#deserialize(JsonElement, Type)} method to create objects for any
   * non-trivial field of the returned object. However, you should never invoke it on the same type
   * passing {@code json} since that will cause an infinite loop (Gson will call your call-back
   * method again).
   *
   * @param json The Json data being deserialized
   * @param typeOfT The type of the Object to deserialize to
   * @param context
   * @return a deserialized object of the specified type typeOfT which is a subclass of {@code T}
   * @throws JsonParseException if json is not in the expected format of {@code typeofT}
   */
  @Override
  public Treasure deserialize(JsonElement json, Type typeOfT, JsonDeserializationContext context)
      throws JsonParseException {
    Gson gson =
        new GsonBuilder()
            .registerTypeAdapter(ItemStack.class, new ItemStackDeserializer())
            .create();
    try {
      Integer id = gson.fromJson("id", Integer.class);
      String loc = gson.fromJson("location", String.class);
      String[] parts = loc.split(",");
      if (parts.length != 6) {
        return null;
      }
      World world = Bukkit.getServer().getWorld(parts[0]);
      double x = Double.parseDouble(parts[1]);
      double y = Double.parseDouble(parts[2]);
      double z = Double.parseDouble(parts[3]);
      float pitch = Float.parseFloat(parts[4]);
      float yaw = Float.parseFloat(parts[5]);
      Location location = new Location(world, x, y, z, pitch, yaw);
      String permission = gson.fromJson("permission", String.class);
      String items = gson.fromJson("items", String.class);
      Type hash = new TypeToken<Map<Integer, ItemStack>>() {}.getType();
      Map<Integer, ItemStack> item = gson.fromJson(items, hash);

      if (permission == null || item == null) {
        return null;
      }

      return new Treasure(id, location, item, permission);

    } catch (Exception e) {
      error("Error deserializing treasure: " + e.getMessage());
    }
    return null;
  }
}
