package me.unurled.sacredrealms.sr.gui.attributes;

import static me.unurled.sacredrealms.sr.utils.Component.comp;
import static me.unurled.sacredrealms.sr.utils.SRPlayerUtils.syncSRToPlayer;

import me.unurled.sacredrealms.sr.components.attributes.Attribute;
import me.unurled.sacredrealms.sr.components.player.PlayerManager;
import me.unurled.sacredrealms.sr.components.player.SRPlayer;
import me.unurled.sacredrealms.sr.managers.Manager;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.ClickType;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.NotNull;
import xyz.xenondevs.inventoryaccess.component.AdventureComponentWrapper;
import xyz.xenondevs.invui.item.Item;
import xyz.xenondevs.invui.item.ItemProvider;
import xyz.xenondevs.invui.item.builder.ItemBuilder;
import xyz.xenondevs.invui.item.impl.AbstractItem;
import xyz.xenondevs.invui.window.Window;

public class AttributeItem extends AbstractItem {

  private final Attribute attribute;

  private final Player targetPlayer;

  public AttributeItem(Attribute attribute, Player target) {
    this.attribute = attribute;
    this.targetPlayer = target;
  }

  /**
   * Gets the {@link ItemProvider}. This method gets called every time a {@link Window} is notified
   * ({@link #notifyWindows()}).
   *
   * @return The {@link ItemProvider}
   */
  @Override
  public ItemProvider getItemProvider() {

    double value = 0;

    if (targetPlayer != null) {
      PlayerManager playerManager = Manager.getInstance(PlayerManager.class);
      SRPlayer sp = playerManager.getPlayer(targetPlayer.getUniqueId());
      if (sp != null) value = sp.getAttribute(attribute);
    }

    ItemBuilder ip =
        switch (attribute) {
          case STRENGTH -> new ItemBuilder(Material.IRON_SWORD).setDisplayName("Strength " + value);
          case AGILITY -> new ItemBuilder(Material.BOW).setDisplayName("Agility " + value);
          case MANA -> new ItemBuilder(Material.ENCHANTING_TABLE).setDisplayName("Mana " + value);
          case HEALTH -> new ItemBuilder(Material.GOLDEN_APPLE).setDisplayName("Health " + value);
          case LUCK ->
              new ItemBuilder(Material.ENCHANTED_GOLDEN_APPLE).setDisplayName("Luck " + value);
          case DEFENSE -> new ItemBuilder(Material.SHIELD).setDisplayName("Defense " + value);
          case CHARISMA ->
              new ItemBuilder(Material.PLAYER_HEAD).setDisplayName("Charisma " + value);
        };

    ip.addLoreLines(new AdventureComponentWrapper(comp("<green><b>+1</b> Left Click to increase")));
    ip.addLoreLines(new AdventureComponentWrapper(comp("<red><b>-1</b> Right Click to decrease")));
    ip.addLoreLines(
        new AdventureComponentWrapper(comp("<gray>Shift + Left Click to increase by 10")));
    ip.addLoreLines(
        new AdventureComponentWrapper(comp("<gray>Shift + Right Click to decrease by 10")));

    return ip;
  }

  /**
   * A method called if the {@link ItemStack} associated to this {@link Item} has been clicked by a
   * player.
   *
   * @param clickType The {@link ClickType} the {@link Player} performed.
   * @param player The {@link Player} who clicked on the {@link ItemStack}.
   * @param event The {@link InventoryClickEvent} associated with this click.
   */
  @Override
  public void handleClick(
      @NotNull ClickType clickType, @NotNull Player player, @NotNull InventoryClickEvent event) {

    if (targetPlayer != null) {
      PlayerManager playerManager = Manager.getInstance(PlayerManager.class);
      SRPlayer sp = playerManager.getPlayer(targetPlayer.getUniqueId());
      if (sp != null) {
        if (clickType == ClickType.LEFT) {
          sp.setAttribute(attribute, sp.getAttribute(attribute) + 1);
        } else if (clickType == ClickType.RIGHT) {
          sp.setAttribute(attribute, sp.getAttribute(attribute) - 1);
        } else if (clickType == ClickType.SHIFT_LEFT) {
          sp.setAttribute(attribute, sp.getAttribute(attribute) + 10);
        } else if (clickType == ClickType.SHIFT_RIGHT) {
          sp.setAttribute(attribute, sp.getAttribute(attribute) - 10);
        }
        playerManager.savePlayer(sp);
        // update player
        syncSRToPlayer(sp, targetPlayer);
        notifyWindows();
      }
    }
  }
}
